// This is -*- C++ -*-
// $Id: DataImport.cc,v 1.3 1998/08/19 16:53:21 hp Exp $

/* DataImport.cc
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "DataImport.h"

string
ColumnSpec::extract(string& s)
{
  char term[3] = "  ";
  if (separator())
    term[1] = separator();

  size_t beg = s.find_first_not_of(' '); // skip over any whitespace
  if (beg == s.npos) {
    s = "";
    return "";
  }

  size_t end = (s[beg] == separator()) ? beg : s.find_first_of(term, beg+1);

  // This can only happen (I hope) if the field was empty (i.e. two terminator
  // characters were separated only by whitespace)
  if (end == beg+1) end = beg;
    
  string ex = (end == s.npos) ? s.substr(beg) : s.substr(beg, end-beg);
  if (end == s.npos) {
    s = "";
  } else {
    end = s.find_first_not_of(' ', end+1);
    s = (end == s.npos) ? "" : s.substr(end);
  }
  return ex;
}

//////////////////////////////////////////////////////////////////////////////

ImportEngine::~ImportEngine()
{
  if (dealloc_strvec_)
    delete strvec_;

  for(int i=0; i<column_count(); ++i)
    delete column(i);
}

void
ImportEngine::preload_istream(istream& in)
{
  dealloc_strvec_ = true;
  strvec_ = new vector<string>;

  char buffer[2048];
  while (in.good()) {
    in.get(buffer,2047, '\n');
    char gobble_newline;
    in.get(gobble_newline);
    if (!in.eof())
      strvec_->push_back(string(buffer));
  }
  strveciter_ = strvec_->begin();
}

vector<DataSet*>
ImportEngine::import()
{
  vector<DataSet*> dsvec;
  // construct our vector of empty DataSet objs
  for(int i=0; i<column_count(); ++i) {
    if (! column(i)->ignore()) {
      DataSet* ds = new DataSet;
      ds->set_label(column(i)->label());
      if (row_count() != -1)
	ds->reserve(row_count());
      dsvec.push_back(ds);
    }
  }

  while (ready()) {
    string s = next_line();
    if (s.length()) { // skip empty lines

      // We don't do anything clever in the event of "short lines".  Our error
      // handling generally sucks.
      vector<DataSet*>::iterator dsi = dsvec.begin();
      for(int i=0; i<column_count(); ++i) {
	string ex = column(i)->extract(s);
	if (! column(i)->ignore()) {
	  double x = atof(ex.c_str());
	  (*dsi)->add(x);
	  ++dsi;
	}
      }

    }
  }

  return dsvec;
}

bool
ImportEngine::ready()
{
  if (in_) {
    return in_->good();
  } else if (strvec_) {
    return strveciter_ != strvec_->end();
  } else
    return false;
}

string
ImportEngine::next_line()
{
  ++row_;

  if (in_) {
    char buffer[2048];
    cin.get(buffer,2047, '\n');

    char gobble_newline;
    cin.get(gobble_newline); 

    return in_->eof() ? string("") : string(buffer);
  } else if (strvec_) {
    return *(strveciter_++);
  } else {
    return string("");
  }
}


// $Id: DataImport.cc,v 1.3 1998/08/19 16:53:21 hp Exp $
