// This is -*- C++ -*-
// $Id: DataImport.h,v 1.2 1998/08/19 16:53:21 hp Exp $

/* DataImport.h
 *
 * Copyright (C) 1998 EMC Capital Management, Inc.
 *
 * Developed by Jon Trowbridge <trow@emccta.com>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#ifndef _INC_DATAIMPORT_H
#define _INC_DATAIMPORT_H

#include <iostream.h>
#include <vector.h>
#include <string>
#include "DataSet.h"

class ColumnSpec {
public:
  ColumnSpec() : label_("Untitled"), sep_('\0'), ignore_(false) { }

  string label() const { return label_; }
  char separator() const { return sep_; }
  bool ignore() const { return ignore_; }

  void set_label(const string& s) { label_ = s; }
  void set_separator(char c) { sep_ = c; }
  void set_ignore(bool b) { ignore_ = b; }

  string extract(string&);

private:
  string label_;
  char sep_;
  bool ignore_;
};

class ImportEngine {
public:
  ImportEngine() : row_(0), in_(0), strvec_(0) { }
  ~ImportEngine();

  void set_istream(istream& i) { in_ = &i; }
  void preload_istream(istream&);

  void set_string_vector(vector<string>& vs) {
    dealloc_strvec_ = false;
    strvec_ = &vs;
    strveciter_ = strvec_->begin();
  }

  int column_count() const { return cols_.size(); }
  ColumnSpec* column(int i) { return cols_[i]; }
  void add_column(ColumnSpec* cs) { cols_.push_back(cs); }

  // Not implemented yet: should peek at the data and fill the ColumnSpec
  // data structures with reasonable default guesses
  void guess_columns(); 

  vector<DataSet*> import();

protected:
  bool ready();
  string next_line();

  int last_row() const { return row_; } // # of line ret by prev next_line()
  int row_count() const { return strvec_ ? (int)(strvec_->size()) : -1; }

private:
  int row_;
  istream* in_;

  bool dealloc_strvec_;
  vector<string>* strvec_;
  vector<string>::iterator strveciter_;

  vector<ColumnSpec*> cols_;
};

#endif // _INC_DATAIMPORT_H

// $Id: DataImport.h,v 1.2 1998/08/19 16:53:21 hp Exp $
